/*  code.c		Jed Margolin 2003
    Serial Tests and Security Code Reader for IDEGP32 Pod
*/

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>


// Serial Port Functions 
extern int init_uart(char* nm, char* speed);
extern int close_uart(void);
extern int send_break10(void);
extern int flush_uart(int out, int in);
extern int wb(const void* data, unsigned len);
extern int rb(void* dest, unsigned len);


void test1(void),test2(void),test3(void),test4(int);
void helpmsg(void),sendit(unsigned);
void sendit(unsigned),sequence(void);
int send_scode(int *,int),read_sp(void),write_data(int,int);
int Iwrite_data(unsigned),Iread_data(void);
unsigned read_data(int);
void get_msg(int);

int conv_arg(char *),asconv(char);
int strfind(unsigned char *, unsigned char *);
int str_to_hex(unsigned char *);

// security codes
int scode1[8] = {0xB0,0xCA,0xB0,0xCA,0xB0,0xCA,0xB0,0x08};
int scode2[8] = {0x11,0x22,0x33,0x44,0x55,0x66,0x77,0x88};
//*************************************************
int main(int argc, char *argv[ ])
{

	char port[16],speed[16];
	int test,address;


	switch (argc)
	{
		case 1:							// default port and speed
			strcpy(port,"1");
			strcpy(speed,"19200");
			test = 3;
			break;

		case 2:
			helpmsg();					// not enough arguments
			exit(0);

		case 3:
			helpmsg();					// not enough arguments
			exit(0);

		case 4:							// arguments for tests 1-3
			strcpy(port,argv[1]);
			strcpy(speed,argv[2]);
			test = atoi(argv[3]);
			if(test==4)
			{
				printf("You forgot to supply an address.\n\n");
				helpmsg();
				exit(0);
			}
			break;

		case 5:							// arguments for test 4
			strcpy(port,argv[1]);
			strcpy(speed,argv[2]);
			test = atoi(argv[3]);
			if (test!=4)
			{
				printf("Number of arguments are not correct for this test\n");
				exit(0);
			}
			address = conv_arg(argv[4]);
			break;

		default:
			helpmsg();
			exit(0);
	}


// open serial port
	if(init_uart(port, speed) < 0)
		exit(1);

	printf("\nPort: %s    Speed: %s Baud   Test: %01d \n\n",port,speed,test);
	switch (test)
	{
	    case 1:
			test1();
			break;

		case 2:
			test2();
			break;

		case 3:
			test3();
			break;

		case 4:
			test4(address);
			break;

		default:
			printf("There are only four tests.\n");
			printf("1 = Send a Continuous Stream of $55.\n");
			printf("2 = Send Incrementing Data and Display the Echo.\n");					
			printf("3 = Send Irrelevant Security Code and Read Back the Security Bytes.\n\n");
			printf("4 = Send Irrelevant Security Code and Read 256 bytes starting at the specified address.\n\n");

			break;	
	
		close_uart();
		return(0);

	}

	return(0);
}
//****************************************************
void helpmsg()
{
	printf("Usage: code <port> <speed> <test#> [address]\n\n");
	printf("Test1 = Send a Continuous Stream of $55.\n");
	printf("Test2 = Send Incrementing Data and Display the Echo.\n");					
	printf("Test3 = Send Irrelevant Security Code and read back the Security bytes.\n");					
	printf("Test4 = Send Irrelevant Security Code and read 256 bytes starting at address.\n\n");

	printf("Example:   code 1 19200 1\n");
	printf("Means:  Port 1, 19200 Baud, Test1\n\n"); 

	printf("Example:   code 1 19200 4 $FE20\n");
	printf("Means:  Port 1, 19200 Baud, Read 256 bytes starting at $FE20\n\n"); 

}
//****************************************************
// Send Continuous Stream of $55 to test serial port connections.
void test1()
{

	unsigned txdata;
	int flag;

	printf("Send a Continuous Stream of $55.\n");
	printf("CTL-BRK to stop\n");

	flag = 1;
	txdata = 0x55;

	while(flag)
	{
	   wb(&txdata,1);
	   Sleep(1);				// 1 ms	
	}

}
//****************************************************
// Send Incrementing Data and Display the Echo.
void test2()
{
	unsigned txdata,rxdata;
	int flag;

	printf("Send Incrementing Data and Display the Echo.\n");					
	printf("CTL-BRK to stop\n\n");

	flush_uart(1,1);		// flush in and out
	flag = 1;
	txdata = 0x00;

	while(flag)
	{
	   wb(&txdata,1);
	   Sleep(10);			// 10 ms	
	   rb(&rxdata,1);
 	   printf("Sent: %02X    Received: %02X \n",txdata,(rxdata&0xFF));
	   txdata = (txdata+1) &0xFF;
	}

}
//****************************************************
// Send Irrelevant Security Code and Read Back the Security Bytes.
void test3()
{
	int n,data,read_address;
	
	printf("\nSend Irrelevant Security Code and Read Back the Security Bytes.\n\n");
	printf("CTL-BRK to stop\n\n");

	printf("\nSend Security Code:   ");
	for(n=0; n<8; n++)  { printf("%02X ",scode2[n]);}
	printf("\n");
	
	send_scode(scode2,0);			// send Security Code

	Sleep(10);					// wait 10ms


	printf("\nReading Security Code and Reset Vector ($FFF6-$FFFF)\n");
	read_address = 0xFFF6;
	for(n=0; n<10; n++)
	{
	   data = read_data(read_address)&0xFF;
  	   printf("\nAddress: %04X   Data: %02X  ",read_address,data);
	   read_address = read_address+0x0001;
	}

	printf("\n\nPod Information:\n");
	get_msg(0xFD00);
	get_msg(0xFD02);
	get_msg(0xFD04);
	get_msg(0xFD06);

}
//****************************************************
// Send Irrelevant Security Code and Read Back data.
void test4(int address)
{
	int n,m,data,read_address;
	
	printf("\nSend Irrelevant Security Code and Read Back 256 bytes\n");
	printf("starting at address $%04X\n\n",address);
	printf("CTL-BRK to stop\n\n");

	printf("\nSend Security Code:   ");
	for(n=0; n<8; n++)  { printf("%02X ",scode2[n]);}
	printf("\n");
	
	send_scode(scode2,0);		// send Security Code

	Sleep(10);					// wait 10ms

	read_address = address;

	for(m=0;m<16;m++)
	{
		printf("\n%04X:    ",read_address);
		for(n=0; n<16; n++)
		{
		   data = read_data(read_address)&0xFF;
		   printf("%02X ",data);
		   read_address = read_address+0x0001;
		}
	}
	printf("\n\n");
}
//****************************************************
// send security code; pflag sends stuff to the screen for debugging
int send_scode(int *scode,int pflag)
{
	int n,result,cflag;
	unsigned charout,charin1,charin2;


// send 8 bytes
	result = 0;
	flush_uart(1,1);		// flush in and out

	printf("\nSending Security Bytes \n");


	for(n=0; n<8; n++)
	{    
		Sleep(1);			// wait 1ms

		charout = scode[n];
		wb(&charout,1);
		rb(&charin1,1);			// hardware echo
		rb(&charin2,1);			// software echo

		cflag = 1;
		if( (charin1 != charout) | (charin2 != charout) )
		{
			cflag = 0;
			result = 1;
		}

		if(pflag)
		{
			printf("\n   Sent: %02X   Received: %02X  %02X   ",charout,charin1,charin2);

			if(cflag)   { printf("ok" ); }
			else        { printf("bad"); }
		}

		charout = charout+0x11;
	}

	rb(&charin2,1);			// break

	return(result);			//	1 = bad
}

//****************************************************
void sendit(unsigned charout)
{
    unsigned charin1,charin2,pflag;

    pflag = 0;
    Sleep(1);				// wait 1ms

	wb(&charout,1);
	Sleep(10);				// 10 ms	
	rb(&charin1,1);			// hardware echo
	rb(&charin2,1);			// software echo

    if(pflag)
    {
	  if( (charin1 != charout) | (charin2 != charout) )
	  {
		 printf("bad echo    ");
	  }
	
	  printf("\n   Sent: %02X   Received: %02X  %02X   ",charout,charin1,charin2);
    }
}
// =====================================================================
// get the data at $FD00 and $FD01; it points to the message
void get_msg(int address)
{
	int adrl,adrh,n,data;
	unsigned read_address;


	adrh = read_data(address)&0xFF;
	adrl=  read_data(address+1)&0xFF;

	read_address = (adrh<<8) + adrl +1;	// the first byte is an lcd display address

	for(n=0;n<32;n++)
	{
		data = read_data(read_address)&0xFF;
		if(data==0xFF)	{ break; }				// end of message
		printf("%c",data);
		read_address++;
	}

	printf("\n");
}
// =====================================================================
// use the Sequence
void sequence()
{
	int data;

	data = read_data(0xFEFB);
	printf("$FEFB: %02X \n",data);

	data = read_data(0xFFF6);
	printf("$FFF6: %02X \n",data);

	data = read_data(0xFFF7);
	printf("$FFF7: %02X \n",data);

	data = read_data(0xFEFB);
	printf("FEFB: %02X \n",data);

	data = read_data(0xFE01);
	printf("SRSR: %02X \n",data);

	data = read_sp();
	printf("SP: %04X \n",data);

	write_data(0x00FE,0x10);
	printf("Write 0x00FE: 0x10\n");

	Iwrite_data(0x00);
	printf("IWrite 0x00FF: 0x00\n");

	return;
}
// ========================================================================
unsigned read_data(int address)
{
	unsigned temp,data;

	sendit(0x4A);

	temp = (address>>8)&0xFF;
	sendit(temp);  			// high byte

	temp=address&0xFF;
	sendit(temp);  			//low byte

	rb(&data,1);
	
	return(data);
}
//----------------------------
int Iread_data(void)
{
	int temp;
	unsigned charin1,charin2;

	sendit(0x1A);

	rb(&charin1,1);
	rb(&charin2,1);

	temp = charin2+(charin1<<8);
	return(temp);
}
//-------------------
int write_data(int address, int data)
{
	unsigned temp;

	sendit(0x49);

	temp = (address>>8)&0xFF;
	sendit(temp);  			// high byte

	temp=address&0xFF;
	sendit(temp);  			//low byte
	Sleep(1);				// wait 1ms

	sendit(data);

	return(data);
}
//--------------------------
int Iwrite_data(unsigned data)
{

	sendit(0x19);			// command

	Sleep(1);				// wait 1ms

    sendit(data);

	return(data);
}
//--------------------------
int read_sp(void)
{
	unsigned charin1,charin2,data;

	sendit(0x0C);

	rb(&charin1,1);
	rb(&charin2,1);

	data = charin2+(charin1<<8);
	return(data);
}
// =====================================================================
int conv_arg(char *line)
{
	unsigned int n;
	int i,start, value;
	int hex = 0;
	char vstr[32];

// Is the address hex?
   start =  strfind("$",line);
   if(start != -1 )
   {
      i = 0;
      for(n=start; n<strlen(line)+1; n++)		vstr[i++] = line[n];
      hex = 1;
   }

   start =  strfind("0x",line);
   if(start != -1 )
   {
      i = 0;
      for(n=start; n<strlen(line)+1; n++)		vstr[i++] = line[n];
      hex = 1;
   }

   start =  strfind("0X",line);
   if(start != -1 )
   {
      i = 0;
      for(n=start; n<strlen(line)+1; n++)		vstr[i++] = line[n];
      hex = 1;
   }

   if(hex == 0)		
   {
      value = atoi(line);
   }

   else
   {
      value = str_to_hex(vstr);
   }

   return(value);
}
// ------------------------------------------------------------------------ 
int str_to_hex(unsigned char *vstr)
{
   unsigned int n;
   int sum = 0;

   for(n=0; n<strlen(vstr); n++)
   {
      sum = sum*16 +  asconv(vstr[n]);        
   }

   return(sum);
}
// ------------------------------------------------------------------------ 
/* Is str contained in line? 
     Start string
     Return index, or 0 if not
*/

int strfind(unsigned char *str, unsigned char *line)
{
	int k,m;
	unsigned int i;
	m = strlen(str);

	for(i=0;i<strlen(line);i++)
	{
      if(str[0] == line[i])
      {	
	     for(k=0; k<m; k++)
		 {
	         if(str[k] != line[i+k])	break;
		 }   
	     if(k == m)  	
		 {
		 //	  printf("first match ends at: %d \n",(i+k) );  
		    return(i+k);
		 }
      }
	}
   return(-1);		// no match
}
// ------------------------------------------------------------------------ 
// Convert ASCII character to integer
int asconv(char val)
{
	if( (val >= 48) && (val <= 57)  )  return(val-48);
	if( (val >= 65) && (val <= 70)  )  return(val-65+10);
	if( (val >= 97) && (val <= 102) )  return(val-97+10);
	return(0);
}
// ========================================================================
